import { connect, Dispatch } from 'react-redux';
import {
  HistoryList,
  StateProps as HistoryStateProps,
  DispatchProps as HistoryDispatchProps,
  Props as SignInProps,
} from './components/HistoryList';
import { push as routerPush } from 'react-router-redux';
import * as asyncRequestActions from '@overflow/commons/redux/action/asyncRequest';
import Probe from '@overflow/probe/api/model/Probe';
import MetaHistoryType from '@overflow/meta/api/model/MetaHistoryType';
import * as readAllByProbeActions from '../redux/action/read_all_by_probe';
import * as readAllByProbeAndTypeActions from '../redux/action/read_all_by_probe_and_type';

export function mapStateToProps(state: any, ownProps?: any): HistoryStateProps {
  return {
    path: ownProps.params,
    histories: state.historyPage,
  };
}

export function mapDispatchToProps(dispatch: Dispatch<any>, ownProps?: any): HistoryDispatchProps {
  return {
    onReadAllByProbe: (probe: Probe, pageNo: string, countPerPage: string) => {
      dispatch(asyncRequestActions.request('HistoryService', 'readAllByProbe', readAllByProbeActions.REQUEST,
                                           JSON.stringify(probe), pageNo, countPerPage));
    },
    onReadAllByProbeAndType: (probe: Probe, type: MetaHistoryType, pageNo: string, countPerPage: string) => {
      dispatch(asyncRequestActions.request('HistoryService', 'readAllByProbeAndType', readAllByProbeAndTypeActions.REQUEST,
                                           JSON.stringify(probe), JSON.stringify(type), pageNo, countPerPage));
    },
  };
}

export default connect(mapStateToProps, mapDispatchToProps)(HistoryList);
