import * as React from 'react';
import { Table, Button, Header, Container, Menu, Icon } from 'semantic-ui-react';
import { ListContainer } from '@overflow/commons/react/component/ListContainer';
import Probe from '@overflow/probe/api/model/Probe';
import History from '@overflow/history/api/model/History';
import Page from '@overflow/commons/api/model/Page';
import MetaHistoryType from '@overflow/meta/api/model/MetaHistoryType';

export interface StateProps {
  histories: Page;
  path: any;
}

export interface DispatchProps {
  onReadAllByProbe?(probe: Probe, pageNo: string, countPerPage: string): void;
  onReadAllByProbeAndType?(probe: Probe, type: MetaHistoryType, pageNo: string, countPerPage: string): void;
}

export type Props = StateProps & DispatchProps;

export interface State {
  page: number;
  isTypedHistory: boolean;
}

export class HistoryList extends React.Component<Props, State> {

  private countPerPage: number = 10;

  constructor(props: Props, context: State) {
    super(props, context);
    this.state = {
      page: 0,
      isTypedHistory: false,
    };
  }

  public componentWillMount(): void {
    if (this.props.path === '/histories') {
      this.setState({
        isTypedHistory: false,
      });
      this.getAllHistory(0);
    } else {
      this.setState({
        isTypedHistory: true,
      });
      this.getTypedHistory(0);
    }
  }

  public getAllHistory(pageNo: number): void {
    let probe: Probe = {
      id: Number(1),
    };
    this.props.onReadAllByProbe(probe, String(pageNo), String(this.countPerPage));
  }

  public getTypedHistory(pageNo: number): void {
    let probe: Probe = {
      id: Number(1),
    };
    let type = this.props.path.split('/')[1];
    console.log(type + '!!!!!!!!!');
    this.props.onReadAllByProbeAndType(probe, this.getMetaType(type), String(pageNo), String(this.countPerPage));
  }

  public getMetaType(typeStr: string): MetaHistoryType {
    let metaType: MetaHistoryType = {
    };
    if (typeStr === 'member') {
      metaType.id = 1;
    } else if (typeStr === 'probe') {
      metaType.id = 2;
    } else if (typeStr === 'discovery') {
      metaType.id = 3;
    } else if (typeStr === 'target') {
      metaType.id = 4;
    } else if (typeStr === 'crawler') {
      metaType.id = 5;
    } else if (typeStr === 'sensor') {
      metaType.id = 6;
    }
    return metaType;
  }

  public handlePaging = (pageNo: number) => {
    if (pageNo === this.state.page) {
      return;
    }
    this.setState({
      page: pageNo,
    });
    let probe: Probe = {
      id: Number(1),
    };
    if (this.state.isTypedHistory) {
      this.getTypedHistory(pageNo);
    } else {
      // this.props.onReadAllByProbe(probe, String(pageNo), String(this.countPerPage));
      this.getAllHistory(pageNo);
    }
  }

  public handlePrevPage = (pn: string) => {
    let p = 0;
    if (pn === 'P') {
      p = this.state.page - 1;
    } else if (pn === 'N') {
      p = this.state.page + 1;
    }
    this.setState({
      page: p,
    });
    let probe: Probe = {
      id: Number(1),
    };
    if (this.state.isTypedHistory) {
      this.getTypedHistory(p);
    } else {
      this.getAllHistory(p);
    }
  }

  public renderPagination = (pageObj: Page): (JSX.Element | JSX.Element[]) => {
    if (pageObj === undefined) {
      return null;
    }
    let keyIdx = 0;
    let totalPage = pageObj.totalPages;
    let currPage = pageObj.number;
    let elems = new Array;
    let prevIndicator = pageObj.first ?
      <Menu.Item as='a' key={keyIdx} icon disabled><Icon name='left chevron' /></Menu.Item> :
      <Menu.Item as='a' key={keyIdx} icon onClick={this.handlePrevPage.bind(this, 'P')}>
        <Icon name='left chevron' />
      </Menu.Item>;
    elems.push(prevIndicator);
    for (let i = 0; i < totalPage; i++) {
      if (i === currPage) {
        elems.push(<Menu.Item key={++keyIdx} active as='a' onClick={this.handlePaging.bind(this, i)}><b>{i + 1}</b></Menu.Item>);
      } else {
        elems.push(<Menu.Item key={++keyIdx} as='a' onClick={this.handlePaging.bind(this, i)}>{i + 1}</Menu.Item>);
      }
    }
    let nextIndicator = pageObj.last ?
      <Menu.Item as='a' key={++keyIdx} icon disabled><Icon name='right chevron' /></Menu.Item> :
      <Menu.Item as='a' key={++keyIdx} icon onClick={this.handlePrevPage.bind(this, 'N')}>
        <Icon name='right chevron' />
      </Menu.Item>;
    elems.push(nextIndicator);

    return <Menu floated='right' pagination>
      {elems}
    </Menu>;
  }

  public render(): JSX.Element {
    let historyList: JSX.Element = (
      <Container fluid>
        <Table celled selectable striped>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell textAlign={'center'}>No.</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Type</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Message</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created At</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created By</Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {this.props.histories && this.props.histories.content ?
              this.props.histories.content.map((history: History, index: number) => (
                <Table.Row key={index} >
                  <Table.Cell textAlign={'center'}>{history.id}</Table.Cell>
                  <Table.Cell textAlign={'center'}>
                    {history.type.name}
                  </Table.Cell>
                  <Table.Cell>
                    {history.message}
                  </Table.Cell>
                  <Table.Cell>
                    {history.createDate}
                  </Table.Cell>
                  <Table.Cell>
                    {history.member.name}
                  </Table.Cell>
                </Table.Row>
              )) : ''}
          </Table.Body>
          <Table.Footer>
            <Table.Row>
              <Table.HeaderCell colSpan='3'>
                {this.renderPagination(this.props.histories)}
              </Table.HeaderCell>
            </Table.Row>
          </Table.Footer>
        </Table>

      </Container>
    );

    return (
      <ListContainer
        contents={historyList}
        data={this.props.histories}
      />
    );

  }

}


